@@ -120,8 +120,7 @@ class LegacyContact(
     def is_friend(self, value: bool) -> None:
         if value == self.is_friend:
             return
-        self.stored.is_friend = value
-        self.commit()
+        self.update_stored_attribute(is_friend=value)
 
     @property
     def added_to_roster(self) -> bool:
@@ -131,8 +130,7 @@ class LegacyContact(
     def added_to_roster(self, value: bool) -> None:
         if value == self.added_to_roster:
             return
-        self.stored.added_to_roster = value
-        self.commit()
+        self.update_stored_attribute(added_to_roster=value)
 
     @property
     def participants(self) -> Iterator["LegacyParticipant"]:
@@ -170,8 +168,7 @@ class LegacyContact(
     def client_type(self, value: ClientType) -> None:
         if self.stored.client_type == value:
             return
-        self.stored.client_type = value
-        self.commit()
+        self.update_stored_attribute(client_type=value)
 
     def _set_logger(self) -> None:
         self.log = logging.getLogger(f"{self.user_jid.bare}:contact:{self}")
@@ -290,13 +287,12 @@ class LegacyContact(
     def name(self, n: Optional[str]) -> None:
         if self.stored.nick == n:
             return
-        self.stored.nick = n
+        self.update_stored_attribute(nick=n)
         self._set_logger()
         if self.is_friend and self.added_to_roster:
             self.xmpp.pubsub.broadcast_nick(
                 user_jid=self.user_jid, jid=self.jid.bare, nick=n
             )
-        self.commit()
         for p in self.participants:
             p.nickname = n or str(self.legacy_id)
 
@@ -361,14 +357,11 @@ class LegacyContact(
         if pronouns:
             vcard["pronouns"]["text"] = pronouns
 
-        self.stored.vcard = str(vcard)
-        self.stored.vcard_fetched = True
+        self.update_stored_attribute(vcard=str(vcard), vcard_fetched=True)
         self.session.create_task(
             self.xmpp.pubsub.broadcast_vcard_event(self.jid, self.user_jid, vcard)
         )
 
-        self.commit()
-
     def get_roster_item(self):
         item = {
             "subscription": self.__get_subscription_string(),
@@ -411,7 +404,7 @@ class LegacyContact(
             # we only broadcast pubsub events for contacts added to the roster
             # so if something was set before, we need to push it now
             self.added_to_roster = True
-            self.send_last_presence()
+            self.send_last_presence(force=True)
 
     async def __broadcast_pubsub_items(self) -> None:
         if not self.is_friend:
