@@ -330,6 +330,7 @@ class BaseGateway(
         self.jid_validator: re.Pattern = re.compile(config.USER_JID_VALIDATOR)
         self.qr_pending_registrations = dict[str, asyncio.Future[Optional[dict]]]()
 
+        self.register_plugins()
         self.__setup_legacy_module_subclasses()
 
         self.get_session_from_stanza: Callable[
@@ -339,7 +340,6 @@ class BaseGateway(
             self._session_cls.from_user
         )
 
-        self.register_plugins()
         self.__register_slixmpp_events()
         self.__register_slixmpp_api()
         self.roster.set_backend(RosterBackend(self))
@@ -390,6 +390,16 @@ class BaseGateway(
         bookmarks_cls = LegacyBookmarks.get_self_or_unique_subclass()
         roster_cls = LegacyRoster.get_self_or_unique_subclass()
 
+        if contact_cls.REACTIONS_SINGLE_EMOJI:  # type:ignore[attr-defined]
+            form = Form()
+            form["type"] = "result"
+            form.add_field(
+                "FORM_TYPE", "hidden", value="urn:xmpp:reactions:0:restrictions"
+            )
+            form.add_field("max_reactions_per_user", value="1", type="number")
+            form.add_field("scope", value="domain")
+            self.plugin["xep_0128"].add_extended_info(data=form)
+
         session_cls.xmpp = self  # type:ignore[attr-defined]
         contact_cls.xmpp = self  # type:ignore[attr-defined]
         muc_cls.xmpp = self  # type:ignore[attr-defined]
@@ -1002,6 +1012,7 @@ SLIXMPP_PLUGINS = [
     "xep_0106",  # JID Escaping
     "xep_0115",  # Entity capabilities
     "xep_0122",  # Data Forms Validation
+    "xep_0128",  # Service Discovery Extensions
     "xep_0153",  # vCard-Based Avatars (for MUC avatars)
     "xep_0172",  # User nickname
     "xep_0184",  # Message Delivery Receipts
