@@ -162,15 +162,12 @@ class AttachmentMixin(TextMessageMixin):
             legacy_file_id=None
             if attachment.legacy_file_id is None
             else str(attachment.legacy_file_id),
-            url=attachment.url,
+            url=attachment.url if config.USE_ATTACHMENT_ORIGINAL_URLS else None,
         )
 
     async def __get_url(
         self, attachment: LegacyAttachment, stored: Attachment
     ) -> tuple[bool, Optional[Path], str]:
-        if attachment.url and config.USE_ATTACHMENT_ORIGINAL_URLS:
-            return False, None, attachment.url
-
         file_name = attachment.name
         content_type = attachment.content_type
         file_path = attachment.path
@@ -221,7 +218,9 @@ class AttachmentMixin(TextMessageMixin):
 
         assert isinstance(file_path, Path)
         if config.FIX_FILENAME_SUFFIX_MIME_TYPE:
-            file_name = str(fix_suffix(file_path, content_type, file_name))
+            file_name, content_type = fix_suffix(file_path, content_type, file_name)
+            attachment.content_type = content_type
+            attachment.name = file_name
 
         if config.NO_UPLOAD_PATH:
             local_path, new_url = await self.__no_upload(
@@ -312,6 +311,50 @@ class AttachmentMixin(TextMessageMixin):
         stored.sfs = str(sfs)
         msg.append(sfs)
 
+    async def __set_sfs_and_sims_without_download(
+        self, msg: Message, attachment: LegacyAttachment
+    ) -> None:
+        assert attachment.url is not None
+
+        if not any(
+            (
+                attachment.content_type,
+                attachment.name,
+                attachment.disposition,
+            )
+        ):
+            return
+
+        sims = self.xmpp.plugin["xep_0385"].stanza.Sims()
+        ref = self.xmpp["xep_0372"].stanza.Reference()
+
+        ref["uri"] = attachment.url
+        ref["type"] = "data"
+        sims["sources"].append(ref)
+        sims.enable("file")
+
+        xep_0447_stanza = self.xmpp.plugin["xep_0447"].stanza
+        sfs = xep_0447_stanza.StatelessFileSharing()
+        url_data = xep_0447_stanza.UrlData()
+        url_data["target"] = attachment.url
+        sfs["sources"].append(url_data)
+        sfs.enable("file")
+
+        if attachment.content_type:
+            sims["file"]["media-type"] = attachment.content_type
+            sfs["file"]["media-type"] = attachment.content_type
+        if attachment.caption:
+            sims["file"]["desc"] = attachment.caption
+            sfs["file"]["desc"] = attachment.caption
+        if attachment.name:
+            sims["file"]["name"] = attachment.name
+            sfs["file"]["name"] = attachment.name
+        if attachment.disposition:
+            sfs["disposition"] = attachment.disposition
+
+        msg.append(sims)
+        msg.append(sfs)
+
     def __send_url(
         self,
         msg: Message,
@@ -325,6 +368,9 @@ class AttachmentMixin(TextMessageMixin):
     ) -> list[Message]:
         msg["oob"]["url"] = uploaded_url
         msg["body"] = uploaded_url
+        if msg.get_plugin("sfs", check=True):
+            msg["fallback"].enable("body")
+            msg["fallback"]["for"] = self.xmpp.plugin["xep_0447"].stanza.NAMESPACE
         if caption:
             m1 = self._send(msg, carbon=carbon, **kwargs)
             m2 = self.send_text(
@@ -465,7 +511,10 @@ class AttachmentMixin(TextMessageMixin):
             new_url = stored.url
         else:
             is_temp, local_path, new_url = await self.__get_url(attachment, stored)
-            if new_url is None:
+            if new_url is None or (
+                local_path is not None and local_path.stat().st_size == 0
+            ):
+                log.warning("Something went wrong with this attachment: %s", attachment)
                 msg["body"] = (
                     "I tried to send a file, but something went wrong. "
                     "Tell your slidge admin to check the logs."
@@ -474,8 +523,13 @@ class AttachmentMixin(TextMessageMixin):
                 return None, [self._send(msg, **kwargs)]
 
         stored.url = new_url
-        thumbnail = await self.__set_sims(msg, new_url, local_path, attachment, stored)
-        self.__set_sfs(msg, new_url, local_path, attachment, stored, thumbnail)
+        if config.USE_ATTACHMENT_ORIGINAL_URLS and attachment.url:
+            await self.__set_sfs_and_sims_without_download(msg, attachment)
+        else:
+            thumbnail = await self.__set_sims(
+                msg, new_url, local_path, attachment, stored
+            )
+            self.__set_sfs(msg, new_url, local_path, attachment, stored, thumbnail)
 
         if self.session is not NotImplemented:
             with self.xmpp.store.session(expire_on_commit=False) as orm:
