@@ -7,9 +7,10 @@ from datetime import datetime, timedelta, timezone
 from mimetypes import guess_extension
 from typing import Collection, Iterator, Optional, Type
 
+import sqlalchemy as sa
 from slixmpp.exceptions import XMPPError
 from slixmpp.plugins.xep_0231.stanza import BitsOfBinary
-from sqlalchemy import Engine, delete, select, update
+from sqlalchemy import Engine, delete, event, select, update
 from sqlalchemy.exc import InvalidRequestError
 from sqlalchemy.orm import Session, attributes, sessionmaker
 
@@ -20,6 +21,7 @@ from .meta import Base
 from .models import (
     ArchivedMessage,
     ArchivedMessageSource,
+    Avatar,
     Bob,
     Contact,
     ContactSent,
@@ -213,6 +215,7 @@ class ContactStore(UpdatedMixin):
     def __init__(self, session: Session) -> None:
         super().__init__(session)
         session.execute(update(Contact).values(cached_presence=False))
+        session.execute(update(Contact).values(caps_ver=None))
 
     @staticmethod
     def add_to_sent(session: Session, contact_pk: int, msg_id: str) -> None:
@@ -589,4 +592,29 @@ class BobStore:
         session.add(row)
 
 
+@event.listens_for(sa.orm.Session, "after_flush")
+def _check_avatar_orphans(session, flush_context):
+    if not session.deleted:
+        return
+
+    potentially_orphaned = set()
+    for obj in session.deleted:
+        if isinstance(obj, (Contact, Room)) and obj.avatar_id:
+            potentially_orphaned.add(obj.avatar_id)
+    if not potentially_orphaned:
+        return
+
+    result = session.execute(
+        sa.delete(Avatar).where(
+            sa.and_(
+                Avatar.id.in_(potentially_orphaned),
+                sa.not_(sa.exists().where(Contact.avatar_id == Avatar.id)),
+                sa.not_(sa.exists().where(Room.avatar_id == Avatar.id)),
+            )
+        )
+    )
+    deleted_count = result.rowcount
+    log.debug(f"Auto-deleted %s orphaned avatars", deleted_count)
+
+
 log = logging.getLogger(__name__)
