@@ -169,8 +169,7 @@ class LegacyMUC(
     def type(self, type_: MucType) -> None:
         if self.type == type_:
             return
-        self.stored.muc_type = type_
-        self.commit()
+        self.update_stored_attribute(muc_type=type_)
 
     @property
     def n_participants(self):
@@ -180,8 +179,7 @@ class LegacyMUC(
     def n_participants(self, n_participants: Optional[int]) -> None:
         if self.stored.n_participants == n_participants:
             return
-        self.stored.n_participants = n_participants
-        self.commit()
+        self.update_stored_attribute(n_participants=n_participants)
 
     @property
     def user_jid(self):
@@ -203,8 +201,7 @@ class LegacyMUC(
     def subject_date(self, when: Optional[datetime]) -> None:
         if self.subject_date == when:
             return
-        self.stored.subject_date = when
-        self.commit()
+        self.update_stored_attribute(subject_date=when)
 
     def __send_configuration_change(self, codes) -> None:
         part = self.get_system_participant()
@@ -222,18 +219,16 @@ class LegacyMUC(
     def user_nick(self, nick: str) -> None:
         if nick == self.user_nick:
             return
-        self.stored.user_nick = nick
-        self.commit()
+        self.update_stored_attribute(user_nick=nick)
 
     def add_user_resource(self, resource: str) -> None:
         stored_set = self.get_user_resources()
         if resource in stored_set:
             return
         stored_set.add(resource)
-        self.stored.user_resources = (
-            json.dumps(list(stored_set)) if stored_set else None
+        self.update_stored_attribute(
+            user_resources=(json.dumps(list(stored_set)) if stored_set else None)
         )
-        self.commit()
 
     def get_user_resources(self) -> set[str]:
         stored_str = self.stored.user_resources
@@ -246,10 +241,9 @@ class LegacyMUC(
         if resource not in stored_set:
             return
         stored_set.remove(resource)
-        self.stored.user_resources = (
-            json.dumps(list(stored_set)) if stored_set else None
+        self.update_stored_attribute(
+            user_resources=(json.dumps(list(stored_set)) if stored_set else None)
         )
-        self.commit()
 
     @asynccontextmanager
     async def lock(self, id_: str) -> AsyncIterator[None]:
@@ -296,7 +290,7 @@ class LegacyMUC(
         self, affiliation: Optional[MucAffiliation] = None
     ) -> AsyncIterator[LegacyParticipantType]:
         await self.__fill_participants()
-        with self.xmpp.store.session(expire_on_commit=False) as orm:
+        with self.xmpp.store.session(expire_on_commit=False, autoflush=False) as orm:
             orm.add(self.stored)
             for db_participant in self.stored.participants:
                 if (
@@ -346,8 +340,7 @@ class LegacyMUC(
     def name(self, n: str | None) -> None:
         if self.name == n:
             return
-        self.stored.name = n
-        self.commit()
+        self.update_stored_attribute(name=n)
         self._set_logger()
         self.__send_configuration_change((104,))
 
@@ -359,8 +352,7 @@ class LegacyMUC(
     def description(self, d: str) -> None:
         if self.description == d:
             return
-        self.stored.description = d
-        self.commit()
+        self.update_stored_attribute(description=d)
         self.__send_configuration_change((104,))
 
     def on_presence_unavailable(self, p: Presence) -> None:
@@ -440,8 +432,7 @@ class LegacyMUC(
         if s == self.subject:
             return
 
-        self.stored.subject = s
-        self.commit()
+        self.update_stored_attribute(subject=s)
         self.__get_subject_setter_participant().set_room_subject(
             s, None, self.subject_date, False
         )
@@ -464,8 +455,7 @@ class LegacyMUC(
         if subject_setter == self.subject_setter:
             return
         assert isinstance(subject_setter, str | None)
-        self.stored.subject_setter = subject_setter
-        self.commit()
+        self.update_stored_attribute(subject_setter=subject_setter)
 
     def __get_subject_setter_participant(self) -> LegacyParticipant:
         if self.subject_setter is None:
@@ -524,6 +514,9 @@ class LegacyMUC(
         if s := self.subject:
             form.add_field("muc#roominfo_subject", value=s)
 
+        if name := self.name:
+            form.add_field("muc#roomconfig_roomname", value=name)
+
         if self._set_avatar_task is not None:
             await self._set_avatar_task
         avatar = self.get_avatar()
@@ -652,7 +645,7 @@ class LegacyMUC(
                 with orm.no_autoflush:
                     orm.refresh(self.stored, ["participants"])
         if not user_participant.is_user:
-            self.log.warning("is_user flag not set participant on user_participant")
+            self.log.warning("is_user flag not set on user_participant")
             user_participant.is_user = True
         user_participant.send_initial_presence(
             user_full_jid,
