@@ -5,6 +5,7 @@ import tempfile
 import unittest
 from contextlib import asynccontextmanager
 from datetime import datetime
+from pathlib import Path
 from unittest.mock import ANY, MagicMock, patch
 
 import pytest
@@ -16,14 +17,27 @@ from slidge.util.types import LegacyAttachment, MessageReference
 
 
 @pytest.fixture(scope="function")
-def attachment(request):
-    class MockResponse:
+def attachment(request, avatar_path):
+    class MockHeadResponse:
         status = 200
 
+    class MockGetResponse:
+        @staticmethod
+        def raise_for_status():
+            pass
+
+        @staticmethod
+        async def read():
+            return avatar_path.read_bytes()
+
     class MockAioHTTP:
         @asynccontextmanager
         async def head(*a, **k):
-            yield MockResponse
+            yield MockHeadResponse
+
+        @asynccontextmanager
+        async def get(*a, **k):
+            yield MockGetResponse
 
     with (
         patch(
@@ -74,9 +88,9 @@ class Base(Shakespeare, AvatarFixtureMixin):
                 use_values=False,
             )
 
-    def _assert_file(self, url="http://url", disposition: str = ''):
+    def _assert_file(self, url="http://url", disposition: str = '', local_path: Path | None = None):
         when = (
-            datetime.fromtimestamp(self.avatar_path.stat().st_mtime)
+            datetime.fromtimestamp((local_path or self.avatar_path).stat().st_mtime)
             .isoformat()
             .replace("+00:00", "Z")
         )
@@ -134,6 +148,9 @@ class Base(Shakespeare, AvatarFixtureMixin):
                 <url>{url}</url>
               </x>
               <body>{url}</body>
+              <fallback xmlns="urn:xmpp:fallback:0" for="urn:xmpp:sfs:0">
+                <body/>
+              </fallback>
             </message>
             """,
             use_values=False,
@@ -283,14 +300,17 @@ class TestAttachmentNoUpload(Base):
 
     def tearDown(self):
         super().tearDown()
-        shutil.rmtree(config.NO_UPLOAD_PATH)
+        try:
+            shutil.rmtree(config.NO_UPLOAD_PATH)
+        except FileNotFoundError:
+            pass
 
-    def __test_basic(self, attachment: LegacyAttachment, url: str):
+    def __test_basic(self, attachment: LegacyAttachment, url: str, local_path: Path | None = None):
         """
         Basic test that file is copied.
         """
         self.run_coro(self.juliet.send_files([attachment]))
-        self._assert_file(url=url)
+        self._assert_file(url=url, local_path=local_path)
 
     def __test_reuse(self, attachment: LegacyAttachment, url: str):
         """
@@ -477,7 +497,110 @@ class TestAttachmentNoUpload(Base):
               <x xmlns="jabber:x:oob">
                 <url>https://url/uuid/uuid/5x5.png</url>
               </x>
+              <fallback xmlns="urn:xmpp:fallback:0"
+                        for="urn:xmpp:sfs:0">
+                <body />
+              </fallback>
             </message>
             """,
             use_values=False,
         )
+
+    def test_url(self):
+        tmp = tempfile.mkdtemp()
+
+        with unittest.mock.patch("tempfile.mkdtemp", return_value=tmp):
+            self.__test_basic(
+                LegacyAttachment(url=self.avatar_url, name="5x5.png"),
+                url="https://url/uuid/uuid/5x5.png",
+                local_path=Path(tmp) / "5x5.png",
+            )
+        shutil.rmtree(tmp)
+
+
+class TestAttachmentOriginalUrl(Base):
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        config.USE_ATTACHMENT_ORIGINAL_URLS = True
+
+
+    @classmethod
+    def tearDownClass(cls):
+        super().tearDownClass()
+        config.USE_ATTACHMENT_ORIGINAL_URLS = False
+
+    def __check(self, url, name=None, content_type=None, disposition=None):
+        disposition = f"disposition='{disposition}'" if disposition else ""
+        # file_sharing = "<file-sharing xmlns='urn:xmpp:sfs:0'" + disposition
+        name = f"<name>{name}</name>" if name else ""
+        content_type = f"<media-type>{content_type}</media-type>" if content_type else ""
+
+        self.send(  # language=XML
+            f"""<message type="chat"
+                     from="juliet@aim.shakespeare.lit/slidge"
+                     to="romeo@montague.lit">
+              <media-sharing xmlns="urn:xmpp:sims:1">
+                <sources>
+                  <reference xmlns="urn:xmpp:reference:0"
+                             uri="{url}"
+                             type="data" />
+                </sources>
+                <file xmlns="urn:xmpp:jingle:apps:file-transfer:5">{content_type} {name}</file>
+              </media-sharing>
+              <file-sharing xmlns="urn:xmpp:sfs:0" {disposition}>
+                <sources>
+                  <url-data xmlns="http://jabber.org/protocol/url-data"
+                            target="{url}" />
+                </sources>
+                <file xmlns="urn:xmpp:file:metadata:0">{content_type} {name}</file>
+              </file-sharing>
+              <x xmlns="jabber:x:oob">
+                <url>{url}</url>
+              </x>
+              <fallback xmlns='urn:xmpp:fallback:0'
+                        for='urn:xmpp:sfs:0'>
+                <body />
+              </fallback>
+              <body>{url}</body>
+            </message>
+            """
+        )
+
+    def __check_no_meta(self, url):
+        self.send(  # language=XML
+            f"""
+            <message type="chat"
+                     from="juliet@aim.shakespeare.lit/slidge"
+                     to="romeo@montague.lit">
+              <x xmlns="jabber:x:oob">
+                <url>{url}</url>
+              </x>
+              <body>{url}</body>
+            </message>
+            """
+        )
+
+    def __send_file(self, url, name=None, content_type=None, disposition=None):
+        self.run_coro(self.juliet.send_file(
+            LegacyAttachment(url=url, name=name, content_type=content_type, disposition=disposition)))
+
+    def test_url_original(self):
+        self.__send_file("prout", name="5x5.png", disposition="inline")
+        self.__check("prout", name="5x5.png", content_type="image/png", disposition="inline")
+
+        self.__send_file("prout", name="5x5.png", content_type="GNAGNA")
+        self.__check("prout", name="5x5.png", content_type="GNAGNA")
+
+        self.__send_file("prout", disposition="inline")
+        self.__check("prout", disposition="inline")
+
+        self.__send_file("prout", name="5x5.png")
+        self.__check("prout", name="5x5.png", content_type="image/png")
+
+        self.__send_file("prout.png")
+        self.__check("prout.png", content_type="image/png")
+
+    def test_no_metadata(self):
+        self.__send_file("prout")
+        self.__check_no_meta("prout")
