@@ -35,10 +35,6 @@ class TestSession(AvatarFixtureMixin, SlidgeTest):
         self.xmpp["xep_0060"].map_node_event(MetaData.namespace, "avatar_metadata")
         register_stanza_plugin(EventItem, MetaData)
 
-    def tearDown(self):
-        super().tearDown()
-        _sessions.clear()
-
     @staticmethod
     def get_romeo_session() -> Session:
         return BaseSession.get_self_or_unique_subclass().from_jid(
