@@ -129,10 +129,6 @@ class TestSession2(AvatarFixtureMixin, SlidgeTest):
             """
         )
 
-    def tearDown(self):
-        super().tearDown()
-        _sessions.clear()
-
     @property
     def romeo_session(self) -> Session:
         return BaseSession.get_self_or_unique_subclass().from_jid(
