@@ -46,10 +46,6 @@ class TestSetNameBeforeFill(AvatarFixtureMixin, SlidgeTest):
         super().setUp()
         self.setup_logged_session(1)
 
-    def tearDown(self):
-        super().tearDown()
-        _sessions.clear()
-
     def test_set_contact_name_before_fill(self):
         contact = self.run_coro(self.romeo.contacts.by_legacy_id("some id"))
         assert contact.name == "some name"
