@@ -47,12 +47,6 @@ unregistered = []
 reactions_received_by_juliet = []
 
 
-class ClearSessionMixin:
-    def tearDown(self):
-        super().tearDown()
-        _sessions.clear()
-
-
 class Gateway(BaseGateway):
     COMPONENT_NAME = "SLIDGE TEST"
 
@@ -163,7 +157,7 @@ class Bookmarks(LegacyBookmarks):
         await self.by_legacy_id("room2")
 
 
-class Base(ClearSessionMixin, SlidgeTest):
+class Base(SlidgeTest):
     plugin = globals()
 
     def setUp(self):
@@ -551,9 +545,10 @@ class TestAimShakespeareBase(Base):
         juliet = self.run_coro(
             session.contacts.by_jid(JID("juliet@aim.shakespeare.lit"))
         )
-        juliet.react("legacy1", "👋")
+        juliet.react("legacy1", ["👋", "👋"])
         msg = self.next_sent()
         assert msg["reactions"]["id"] == "legacy1"
+        assert len(msg["reactions"]) == 1
         for r in msg["reactions"]:
             assert r["value"] == "👋"
         reactions_received_by_juliet.clear()
@@ -1202,7 +1197,7 @@ class TestAimShakespeareBase(Base):
         )
 
 
-class TestPrivilege(ClearSessionMixin, SlidgeTest):
+class TestPrivilege(SlidgeTest):
     plugin = globals()
 
     def setUp(self):
@@ -1294,7 +1289,7 @@ class TestPrivilege(ClearSessionMixin, SlidgeTest):
         )
 
 
-class TestContact(ClearSessionMixin, SlidgeTest):
+class TestContact(SlidgeTest):
     plugin = globals()
 
     def setUp(self):
@@ -1731,7 +1726,7 @@ class TestContact(ClearSessionMixin, SlidgeTest):
         )
 
 
-class TestCarbon(ClearSessionMixin, SlidgeTest):
+class TestCarbon(SlidgeTest):
     plugin = globals()
 
     def setUp(self):
@@ -1841,6 +1836,10 @@ class TestCarbon(ClearSessionMixin, SlidgeTest):
                         </file>
                       </media-sharing>
                     </reference>
+                    <fallback xmlns="urn:xmpp:fallback:0"
+                              for="urn:xmpp:sfs:0">
+                      <body />
+                    </fallback>
                     <file-sharing xmlns="urn:xmpp:sfs:0">
                       <sources>
                         <url-data xmlns="http://jabber.org/protocol/url-data"
